package cn.iocoder.yudao.module.infra.controller.app.file;

import cn.hutool.core.io.IoUtil;
import cn.hutool.core.io.file.FileNameUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.common.util.servlet.ServletUtils;
import cn.iocoder.yudao.module.infra.service.file.FileService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestPart;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.Objects;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.module.infra.enums.ErrorCodeConstants.FILE_IS_EMPTY;

/**
 * @author Administrator
 */
@Slf4j
@Validated
@RestController
@RequestMapping("file")
@Api(tags = "文件存储 - 文件上传")
public class AppFileController {
    @Resource
    private FileService fileService;

    @PostMapping("upload")
    @ApiOperation(value = "文件上传")
    @ApiImplicitParams({@ApiImplicitParam(paramType = "query", dataType="file", name = "file", value = "文件附件", required = true)})
    public CommonResult<String> upload(@RequestPart("file") MultipartFile file) throws Exception {
        if (Objects.isNull(file)) { throw exception(FILE_IS_EMPTY); }
        // 获取文件后缀
        String fix = FileNameUtil.getSuffix(file.getOriginalFilename());
        String path = DateUtils.getDir() + IdUtil.objectId() + (StrUtil.isBlank(fix) ? StrUtil.EMPTY : StrUtil.DOT + fix);
        return success(fileService.createFile(path, file.getBytes()));
    }

    @PostMapping("org-name/up")
    @ApiOperation("上传文件(源文件名称返回)")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "query", dataType="file", name = "file", value = "文件附件", required = true, dataTypeClass = MultipartFile.class)
    })
    public CommonResult<String> upFile(@RequestPart("file") MultipartFile file) throws Exception {
        String fix = FileNameUtil.getSuffix(file.getOriginalFilename());
        String path = DateUtils.getDir() + FileNameUtil.getPrefix(file.getOriginalFilename()) + (StrUtil.isBlank(fix) ? StrUtil.EMPTY : StrUtil.DOT + fix);
        return success(fileService.createFile(path, IoUtil.readBytes(file.getInputStream())));
    }

    @GetMapping("/{configId}/get/{path}")
    @ApiOperation("文件查看")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "configId", value = "配置编号",  required = true, dataTypeClass = Long.class),
        @ApiImplicitParam(name = "path", value = "文件路径", required = true, dataTypeClass = String.class)
    })
    public void getFileContent(HttpServletResponse response,
                               @PathVariable("configId") Long configId,
                               @PathVariable("path") String path) throws Exception {
        byte[] content = fileService.getFileContent(configId, path);
        if (content == null) {
            log.warn("[getFileContent][configId({}) path({}) 文件不存在]", configId, path);
            response.setStatus(HttpStatus.NOT_FOUND.value());
            return;
        }
        ServletUtils.writeAttachment(response, path, content);
    }
}
